' *******************************************************************************************************
' *                                                                                                     *
' *     AiChip_Basic_Tv_001.spin                                                                        *
' *                                                                                                     *
' *******************************************************************************************************
' *                                                                                                     *
' *     BUG : USING AiChip_SmallFontMedium_XXX.spin NOT AiChip_SmallFont_XXX.spin BECAUSE ...           *
' *                                                                                                     *
' *     BUG : SCREEN COLOUR HANDLING DOES NOT WORK IN AiChip_SmallFont_XXX.spin                         *
' *                                                                                                     *
' *******************************************************************************************************

PUB Version
  result := String( "AiChip Basic Tv 001" )
  
OBJ

  tv_const                      : "AiChip_Basic_Tv_Constants_001"

  tv_LargeFont                  : "AiChip_Basic_Tv_LargeFont_001"       '  320 longs for buffer

CON

' .-----------------------------------------------------------------------------------------------------.
' |                                                                                                     |
' |     TV Modes                                                                                        |
' |                                                                                                     |
' `-----------------------------------------------------------------------------------------------------'

  TV_MODE_NTSC                  = tv_const#TV_MODE_NTSC
  TV_MODE_PAL                   = tv_const#TV_MODE_PAL

  TV_MODE_LARGE_FONT            = tv_const#TV_MODE_LARGE_FONT
  TV_MODE_SMALL_FONT            = tv_const#TV_MODE_SMALL_FONT

  TV_MODE_FONT_0                = tv_const#TV_MODE_FONT_0
  TV_MODE_FONT_1                = tv_const#TV_MODE_FONT_1
  TV_MODE_FONT_2                = tv_const#TV_MODE_FONT_2
  TV_MODE_FONT_3                = tv_const#TV_MODE_FONT_3

  TV_MODE_FAST_UPDATE           = tv_const#TV_MODE_FAST_UPDATE
  TV_MODE_FLICKER_FREE          = tv_const#TV_MODE_FLICKER_FREE

  TV_MODE_COLOR                 = tv_const#TV_MODE_COLOR
  TV_MODE_COLOUR                = tv_const#TV_MODE_COLOUR
  TV_MODE_MONOCHROME            = tv_const#TV_MODE_MONOCHROME
  
  TV_MODE_INTERLACED            = tv_const#TV_MODE_INTERLACED
  TV_MODE_NON_INTERLACED        = tv_const#TV_MODE_NON_INTERLACED

  TV_MODE_COMPOSITE             = tv_const#TV_MODE_COMPOSITE
  TV_MODE_BASEBAND              = tv_const#TV_MODE_BASEBAND
  TV_MODE_BROADCAST             = tv_const#TV_MODE_BROADCAST

  TV_MODE_FORCED                = tv_const#TV_MODE_FORCED
  
' .-----------------------------------------------------------------------------------------------------.
' |                                                                                                     |
' |     Colour Definitions                                                                              |
' |                                                                                                     |
' `-----------------------------------------------------------------------------------------------------'
    
  RGB_BLACK                     = $02 
  RGB_GREY                      = $03
  RGB_WHITE                     = $06
  RGB_BRIGHT_WHITE              = $07
  RGB_CYAN                      = $3D
  RGB_LIGHT_CYAN                = $3E 
  RGB_LIGHT_GREEN               = $5E 
  RGB_LIGHT_YELLOW              = $8E 
  RGB_LIGHT_RED                 = $CE
  RGB_LIGHT_MAGENTA             = $FE 

  RGB_RED                       = $BC          
  RGB_YELLOW                    = $8D
  RGB_GREEN                     = $5C
  RGB_BLUE                      = $1C
  RGB_MAGENTA                   = $FC
  RGB_LIGHT_BLUE                = $1E

' .-----------------------------------------------------------------------------------------------------.
' |                                                                                                     |
' |     Screen Buffer and Colour Pallette sizes required by driver                                      |
' |                                                                                                     |
' `-----------------------------------------------------------------------------------------------------'

  LARGEFONT_SCREEN_SIZE_LONGS   = tv_LargeFont#SCREEN_SIZE_LONGS

  SCREEN_SIZE_LONGS             = LARGEFONT_SCREEN_SIZE_LONGS 

  LARGEFONT_PALETTE_SIZE_LONGS  = tv_LargeFont#COLOURPALETTE_SIZE_LONGS

  COLOURPALETTE_SIZE_LONGS      = LARGEFONT_PALETTE_SIZE_LONGS 

VAR

  long screenBuffer[SCREEN_SIZE_LONGS]

  ' This is not passed down as a pointer yet ...
  ' long colourPalette[COLOURPALETTE_SIZE_LONGS]

  byte rows
  byte columns

  byte row
  byte column
  byte colour

  word mode
  
' *******************************************************************************************************
' *                                                                                                     *
' *     Device Handling Routines                                                                        *
' *                                                                                                     *
' *******************************************************************************************************

PUB Open( setBasePin, setMode )

  setMode |=   TV_MODE_FORCED
  setMode &= ! TV_MODE_BROADCAST

  setMode := tv_LargeFont.ForceAttributes( setMode ) 
  
  if mode <> setMode

    Close
     
    mode    := tv_LargeFont.Open( setBasePin, setMode, @screenBuffer, @PALETTE )
    rows    := tv_LargeFont.GetRowCount
    columns := tv_LargeFont.GetColumnCount

  return mode
  
PUB Close

  if mode
    tv_LargeFont.Close
    mode := 0

PUB GetMode

  result := mode

' *******************************************************************************************************
' *                                                                                                     *
' *     Text Handling Routines                                                                          *
' *                                                                                                     *
' *******************************************************************************************************

PUB Out( b )

  case b
    $00   : Cls
    $01   : Home
    $08   : BackSpace
    $09   : Tab
    $0A   : 
    $0D   : CrLf
    "0"   : RawOut( "O" )
    other : RawOut( b )
  
PUB RawOut( b )

  if column => columns
    CrLf
  if row => rows
    Scroll
  PokeChar( row, column, colour, b )
  column++

PUB CrLf

  column := 0
  if row++ > rows
    Scroll
    
PUB Cls

  tv_LargeFont.Cls
  colour := 0
  Home

PUB Home

  column := row := 0

PUB Tab

  TabTo( 9 )
  
PUB TabTo( n )

  repeat
    RawOut( " " )
  while ( column // n )

PUB BackSpace

  if column
    column--
  else
    column := 0
    if row
      row--
  PokeChar(row,column,colour," ")

PUB At( argRow, argColumn )

  AtRow( argRow )
  AtColumn( argColumn )
  
PUB AtRow( argRow )

  row := ( 1 #> argRow <# rows ) - 1

PUB AtColumn( argColumn )

  column := ( 1 #> argColumn <# columns ) - 1

PRI Scroll

  repeat while row => rows
    tv_LargeFont.Scroll
    row--

PRI PokeChar(argRow,argCol,useColour,c)

  tv_LargeFont.PokeChar(argRow,argCol,useColour,c)

' *******************************************************************************************************
' *                                                                                                     *
' *     Attribute Handling for Video Drivers                                                            *
' *                                                                                                     *
' *******************************************************************************************************

PUB GetColourCount

  result := tv_LargeFont.GetColourCount

PUB GetColumnCount

  result := columns
    
PUB GetRowCount

  result := rows
  
PUB GetCurrentColumn

  result := column

PUB GetCurrentRow

  result := row

' *******************************************************************************************************
' *                                                                                                     *
' *     Colour Palette Handling for Video Drivers                                                       *
' *                                                                                                     *
' *******************************************************************************************************

PUB SetPalette( ptrToPalette )

  tv_LargeFont.SetPalette( ptrToPalette )
              
PUB SetPaletteColour( colorIdx, fore, back )

  tv_LargeFont.SetPaletteColour( colorIdx, fore, back )

PUB SetColour( useColour )

  colour := useColour

DAT

' *******************************************************************************************************
' *                                                                                                     *
' *     Colour Palette for Video Drivers                                                                *
' *                                                                                                     *
' *******************************************************************************************************

' Basic QB-style colours
'
'       0 White                 8  Grey
'       1 Blue                  9  Light Blue
'       2 Green                 10 Light Green
'       3 Cyan                  11 Light Cyan
'       4 Red                   12 Light Red
'       5 Magenta               13 Light Magenta
'       6 Yellow                14 Light Yellow
'       7 Bright White          15 White on Red
        
PALETTE byte    RGB_BLACK              ' Background

        byte    RGB_WHITE              ' 0
        byte    RGB_BLUE               ' 1
        byte    RGB_GREEN              ' 2  
        byte    RGB_CYAN               ' 3  
        byte    RGB_RED                ' 4  
        byte    RGB_MAGENTA            ' 5  
        byte    RGB_YELLOW             ' 6  
        byte    RGB_BRIGHT_WHITE       ' 7
          
        byte    RGB_GREY               ' 8 
        byte    RGB_LIGHT_BLUE         ' 9
        byte    RGB_LIGHT_GREEN        ' 10
        byte    RGB_LIGHT_CYAN         ' 11
        byte    RGB_LIGHT_RED          ' 12
        byte    RGB_LIGHT_MAGENTA      ' 13
        byte    RGB_LIGHT_YELLOW       ' 14

        byte    RGB_RED                ' 15 - Background with foreground colour 0

' *******************************************************************************************************
' *                                                                                                     *
' *     End of AiChip_TvText_003.spin                                                                   *
' *                                                                                                     *
' *******************************************************************************************************